//
//  SMKishochoParser.m
//  KishochoApp
//
//  Created by Cyan K on 2007/11/01.
//  cyanworks.net
//

#import "SMKishochoParser.h"

@implementation SMKishochoParser

#pragma mark Implementation
// Read weather data from a DOMDocument
- (NSArray *)parseDocument:(DOMDocument *)document withMode:(unsigned int)mode
{
	NSMutableArray	*nodeArray = [NSMutableArray array];
	DOMNodeList		*nodeList;
	id				obj, childObj, grandChildObj;
	int				i, tr;
	
	switch (mode) {
		case SM3daysWeather:
			// Read 3days data
			#ifdef DEBUG
				NSLog(@"Parsing 3days data.");
			#endif
			nodeList = [document getElementsByTagName:@"TABLE"];
			for (i = 0; i < [nodeList length]; i++) {
				obj = nil;
				#ifdef DEBUG
					NSLog(@"OBJ %d: %@", i, [[nodeList item:i] className]);
				#endif
				if ([[[nodeList item:i] className] isEqualToString:@"forecast"]) {
					obj = [nodeList item:i];
					break;
				}
			}
			if (!obj) {
				NSLog(@"Element not found : table.forecast");
				return nil;
			}

			for (i=0; i<[[obj childNodes] length]; i++) {
				childObj = nil;
				#ifdef DEBUG
					NSLog(@"childObj %d: %@", i, [[[obj childNodes] item:i] nodeName]);
				#endif
				if ([[[[obj childNodes] item:i] nodeName] isEqualToString:@"TBODY"]) {
					childObj = [[obj childNodes] item:i];
				}
			}
			if (!childObj) {
				NSLog(@"Element not found : table.forecast tbody");
				return nil;
			}

			tr = 0;
			for (i = 0; i < [[childObj childNodes] length]; i++) {
				grandChildObj = [[childObj childNodes] item:i];
				#ifdef DEBUG
					NSLog(@"grandChildObj %d %d : %@", i, tr, [grandChildObj nodeName]);
				#endif
				if ([[grandChildObj nodeName] isEqualToString:@"TR"]) {
					if (   tr==1  || tr==2  || tr==3
						|| tr==5  || tr==6  || tr==7
						|| tr==9  || tr==10 || tr==11
						|| tr==13 || tr==14 || tr==15
						|| tr==17 || tr==18 || tr==19 ) {
						[nodeArray addObject:grandChildObj];
					}
					tr++;
				}
			}
			if ([nodeArray count] < 3) {
				NSLog(@"Element not found : table.forecast tbody tr");
				return nil;
			}
			#ifdef DEBUG
				for (i = 0; i < [nodeArray count]; i++)
					NSLog (@"%d:%@", i, [[nodeArray objectAtIndex:i] innerText]);
			#endif
			return [self parseData3:nodeArray];
			break;

		case SM7daysWeather:
			// Read 7days data
			#ifdef DEBUG
				NSLog(@"Parsing 7days data.");
			#endif
			nodeList = [document getElementsByTagName:@"TABLE"];
			for (i = 0; i < [nodeList length]; i++) {
				obj = nil;
				#ifdef DEBUG
					NSLog(@"OBJ %d: %@", i, [[nodeList item:i] className]);
				#endif
				if ([[[nodeList item:i] className] isEqualToString:@"forecast-top"]) {
					obj = [nodeList item:i];
					break;
				}
			}
			if (!obj) {
				NSLog(@"Element not found : table.forecast-top");
				return nil;
			}

			for (i = 0; i < [[obj childNodes] length]; i++) {
				childObj = nil;
				#ifdef DEBUG
					NSLog(@"childObj %d: %@", i, [[[obj childNodes] item:i] nodeName]);
				#endif
				if ([[[[obj childNodes] item:i] nodeName] isEqualToString:@"TBODY"])
					childObj = [[obj childNodes] item:i];
			}
			if (!childObj) {
				NSLog(@"Element not found : table.forecast-top tbody");
				return nil;
			}

			tr = 0;
			for (i = 0; i < [[childObj childNodes] length]; i++) {
				grandChildObj = [[childObj childNodes] item:i];
				#ifdef DEBUG
					NSLog(@"grandChildObj %d %d : %@", i, tr, [grandChildObj nodeName]);
				#endif
				if ([[grandChildObj nodeName] isEqualToString:@"TR"]) {
					[nodeArray addObject:grandChildObj];
					tr++;
				}
			}
			if ([nodeArray count] < 5) {
				NSLog(@"Element not found : table.forecast-top tbody tr");
				return nil;
			}
			#ifdef DEBUG
				for (i = 0; i < [nodeArray count]; i++)
					NSLog (@"%d:%@", i, [[nodeArray objectAtIndex:i] innerText]);
			#endif
			return [self parseData7:nodeArray];
			break;

		default:
			NSLog(@"invalid mode:%d", mode);
			return nil;
			break;
	}
	return nil;
}

- (NSArray *)parseData3:(NSMutableArray *)nodeArray
{
	id				obj, childObj, grandchildObj, grandgrandchildObj;
	int				i, j, tr, thd;
	NSString		*date;
	NSString		*weatherIcon;
	NSString		*weather;
	NSString		*windAndWave;
	NSMutableArray	*rProv;
	NSString		*tempL1;
	NSString		*tempL2;
	NSString		*tempL3;
	NSString		*tempH1;
	NSString		*tempH2;
	NSString		*tempH3;
	NSMutableArray	*sectionData;
	NSMutableArray	*loadedData = [NSMutableArray array];

	#ifdef DEBUG
		NSLog(@"Parsing 3days data.");
	#endif
	sectionData = [NSMutableArray array];

	for (tr = 0; tr < [nodeArray count]; tr++) {
		obj = [nodeArray objectAtIndex:tr];
		date = @"n/a";
		weatherIcon = @"n/a";
		weather = @"n/a";
		windAndWave = @"n/a";
		rProv = [NSMutableArray array];
		tempL1 = @"n/a";
		tempL2 = @"n/a";
		tempL3 = @"n/a";
		tempH1 = @"n/a";
		tempH2 = @"n/a";
		tempH3 = @"n/a";

		thd = 0;
		for (i = 0; i < [[obj childNodes] length]; i++) {
			childObj = [[obj childNodes] item:i];
			#ifdef DEBUG
				NSLog(@"childObj %d %d: %@", i, thd, [childObj nodeName]);
			#endif
			if ([[childObj nodeName] isEqualToString:@"TH"] || [[childObj nodeName] isEqualToString:@"TD"]) {
				switch (thd) {
					// TH : date and weather icon
					case 0:
						if (![[childObj className] isEqualToString:@"weather"]) {
							NSLog(@"Element not found : tr .weather");
							return nil;
						}
						date = [[childObj innerText] stringByExtractingNumber];
						for (j=0; j<[[childObj childNodes] length]; j++) {
							grandchildObj = [[childObj childNodes] item:j];
							if ([[grandchildObj nodeName] isEqualToString:@"IMG"]) {
								weatherIcon = [grandchildObj getAttribute:@"src"];
								weatherIcon = [weatherIcon stringWeatherForCode];
							}
						}
						break;

					// TD(1): weather and wind and wave
					case 1:
						if (![[childObj className] isEqualToString:@"info"]) {
							NSLog(@"Element not found : tr .info");
							return nil;
						}
						#ifdef DEBUG
							NSLog(@"childObj childNodes length: %d", [[childObj childNodes] length]);
						#endif
						grandchildObj = [[childObj childNodes] item:0];
						if (grandchildObj)
							if ([[grandchildObj nodeName] isEqualToString:@"#text"])
								if (![[grandchildObj nodeValue] isEqualToString:@""])
									weather = [[grandchildObj nodeValue] stringByDeletingSpaces];
						#ifdef DEBUG
							NSLog(@"weather: %@", weather);
						#endif
						grandchildObj = [[childObj childNodes] item:2];
						if (grandchildObj)
							if ([[grandchildObj nodeName] isEqualToString:@"#text"])
								if (![[grandchildObj nodeValue] isEqualToString:@""])
									windAndWave = [[grandchildObj nodeValue] stringByDeletingSpaces];
						#ifdef DEBUG
							NSLog(@"windAndWave: %@", windAndWave);
						#endif
						break;

					// TD(2): probability of precipitation
					case 2:
						if (![[childObj className] isEqualToString:@"rain"]) {
							NSLog(@"Element not found : tr .rain");
							return nil;
						}
						for (j = 0; j < 4; j++) {
							grandchildObj = [[childObj getElementsByTagName:@"TR"] item:(j)];
							grandgrandchildObj = [[grandchildObj getElementsByTagName:@"TD"] item:(1)];
							if (grandgrandchildObj != NULL)
								[rProv addObject:[[grandgrandchildObj innerText] stringByExtractingNumber]];
							else
								[rProv addObject:@"n/a"];
						}
						break;

					// TD(3): temperature
					case 3:
						if (![[childObj className] isEqualToString:@"temp"]) {
							NSLog(@"Element not found : tr .temp");
							return nil;
						}
						for (j = 0; j < [[childObj getElementsByTagName:@"TD"] length]; j++) {
							grandchildObj = [[childObj getElementsByTagName:@"TD"] item:(j)];
							if ([[grandchildObj className] isEqualToString:@"min"]) {
								if ([tempL1 isEqualToString:@"n/a"]) {
									tempL1 = [[grandchildObj innerText] stringByExtractingNumber];
								} else if ([tempL2 isEqualToString:@"n/a"]) {
									tempL2 = [[grandchildObj innerText] stringByExtractingNumber];
								} else if ([tempL3 isEqualToString:@"n/a"]) {
									tempL3 = [[grandchildObj innerText] stringByExtractingNumber];
								}
							}
							if ([[grandchildObj className] isEqualToString:@"max"]) {
								if ([tempH1 isEqualToString:@"n/a"]) {
									tempH1 = [[grandchildObj innerText] stringByExtractingNumber];
								} else if ([tempH2 isEqualToString:@"n/a"]) {
									tempH2 = [[grandchildObj innerText] stringByExtractingNumber];
								} else if ([tempH3 isEqualToString:@"n/a"]) {
									tempH3 = [[grandchildObj innerText] stringByExtractingNumber];
								}
							}
						}
						break;
				}
				thd++;
			}
		}

		[sectionData addObject:[NSArray arrayWithObjects:date,
								weatherIcon, weather, windAndWave, rProv,
								tempL1, tempH1, tempL2, tempH2, tempL3, tempH3, nil]];
		if ([sectionData count] == 3) {
			[loadedData addObject:sectionData];
			sectionData = [NSMutableArray array];
		}
	}
	return loadedData;
}

- (NSArray *)parseData7:(NSMutableArray *)nodeArray
{
	id				obj, childObj, grandchildObj;
	int				i, j, k, th, td;
	BOOL			pct;
	DOMNodeList		*nodeList;
	NSString		*pattern;
	NSString		*date;
	NSString		*weatherIcon;
	NSString		*rProv;
	NSString		*tempL;
	NSString		*tempH;
	NSMutableArray	*date7;
	NSMutableArray	*weatherIcon7;
	NSMutableArray	*rProv7;
	NSMutableArray	*tempL7;
	NSMutableArray	*tempH7;
	NSMutableArray	*sectionData;
	NSMutableArray	*loadedData = [NSMutableArray array];

	#ifdef DEBUG
		NSLog(@"Parsing 7days data: %d", [nodeArray count]);
	#endif
	// TR: date7
	obj = [nodeArray objectAtIndex:0];
	date7 = [NSMutableArray array];
	for (i = 1; i < [[obj childNodes] length]; i++) {
		childObj = [[obj childNodes] item:i];
		if ([[childObj nodeName] isEqualToString:@"TH"]) {
			date = [[childObj innerText] stringByExtractingNumber];
			if (![date isEqualToString:@"n/a"]) {
				[date7 addObject:date];
			}
		}
	}

	// Check outline of table
	pattern = @"";
	for (i = 1; i < [nodeArray count] - 1; i++) {
		obj = [nodeArray objectAtIndex:i];

		// Count th or td elements
		th = td = 0;
		pct = NO;
		th = [[obj getElementsByTagName:@"TH"] length];
		td = [[obj getElementsByTagName:@"TD"] length];
		if (0 < [[obj innerHTML] rangeOfString:@"\%"].length)
			pct = YES;

		#ifdef DEBUG
			NSLog(@"nodeArray[%d]: %d, %d", i, th, td);
		#endif
		if ((th == 1) && (td == 7)) {
			pattern = [pattern stringByAppendingString:@"W"];	// weather
		} else if ((th == 1) && (td == 8)) {
			pattern = [pattern stringByAppendingString:@"H"];	// temp High
			pattern = [pattern stringByAppendingString:@"L"];	// temp Low
			i++;
		} else if (pct) {
			pattern = [pattern stringByAppendingString:@"P"];	// probability of precipitation
		} else {
			[nodeArray removeObjectAtIndex:i];					// Remove (reliability or others)
			i--;
		}
	}
	#ifdef DEBUG
		NSLog(@"pattern: %@", pattern);
	#endif
	if (!( [pattern isEqualToString:@"WPHL"]
		|| [pattern isEqualToString:@"WPHLWP"]
		|| [pattern isEqualToString:@"WPHLWPHL"]
		|| [pattern isEqualToString:@"WPHLWPWP"]
		|| [pattern isEqualToString:@"WPHLWPWPHL"]
		|| [pattern isEqualToString:@"WPHLWPHLWP"]
		|| [pattern isEqualToString:@"WPHLWPHLWPHL"]
		|| [pattern isEqualToString:@"WPHLWPHLWPHLWPHL"]
		|| [pattern isEqualToString:@"WPWPHL"]
		|| [pattern isEqualToString:@"WPWPWPHL"]
		))
	{
		NSLog(@"Unknown table: week");
		return nil;
	}

	for (i = 1; i <= 16; i++) {
		// weatherIcon7
		if (   ([pattern isEqualToString:@"WPHL"]         &&  (i == 1)                          )
			|| ([pattern isEqualToString:@"WPHLWP"]       && ((i == 1) || (i == 5)             ))
			|| ([pattern isEqualToString:@"WPHLWPHL"]     && ((i == 1) || (i == 5)             ))
			|| ([pattern isEqualToString:@"WPHLWPWP"]     && ((i == 1) || (i == 5) || (i == 7) ))
			|| ([pattern isEqualToString:@"WPHLWPWPHL"]   && ((i == 1) || (i == 5) || (i == 7) ))
			|| ([pattern isEqualToString:@"WPHLWPHLWP"]   && ((i == 1) || (i == 5) || (i == 9) ))
			|| ([pattern isEqualToString:@"WPHLWPHLWPHL"] && ((i == 1) || (i == 5) || (i == 9) ))
			|| ([pattern isEqualToString:@"WPHLWPHLWPHLWPHL"] && ((i == 1) || (i == 5) || (i == 9) || (i ==13) ))
			|| ([pattern isEqualToString:@"WPWPHL"]       && ((i == 1) || (i == 3)             ))
			|| ([pattern isEqualToString:@"WPWPWPHL"]     && ((i == 1) || (i == 3) || (i == 5) ))
			)
		{
			#ifdef DEBUG
				NSLog(@"%d weatherIcon", i);
			#endif
			obj = [nodeArray objectAtIndex:i];
			weatherIcon7 = [NSMutableArray array];
			nodeList = [obj getElementsByTagName:@"TD"];
			for (j = 0; j < 7; j++) {
				childObj = [nodeList item:j];
				weatherIcon = @"n/a";
				for (k = 0; k < [[childObj childNodes] length]; k++) {
					grandchildObj = [[childObj childNodes] item:k];
					if ([[grandchildObj nodeName] isEqualToString:@"IMG"]) {
						weatherIcon = [grandchildObj getAttribute:@"src"];
						weatherIcon = [weatherIcon stringWeatherForCode];
					}
				}
				[weatherIcon7 addObject:weatherIcon];
			}
		}
		
		// rProv7
		if (   ([pattern isEqualToString:@"WPHL"]         &&  (i == 2)                          )
			|| ([pattern isEqualToString:@"WPHLWP"]       && ((i == 2) || (i == 6)             ))
			|| ([pattern isEqualToString:@"WPHLWPHL"]     && ((i == 2) || (i == 6)             ))
			|| ([pattern isEqualToString:@"WPHLWPWP"]     && ((i == 2) || (i == 6) || (i == 8) ))
			|| ([pattern isEqualToString:@"WPHLWPWPHL"]   && ((i == 2) || (i == 6) || (i == 8) ))
			|| ([pattern isEqualToString:@"WPHLWPHLWP"]   && ((i == 2) || (i == 6) || (i == 10)))
			|| ([pattern isEqualToString:@"WPHLWPHLWPHL"] && ((i == 2) || (i == 6) || (i == 10)))
			|| ([pattern isEqualToString:@"WPHLWPHLWPHLWPHL"] && ((i == 2) || (i == 6) || (i == 10) || (i == 14)))
			|| ([pattern isEqualToString:@"WPWPHL"]       && ((i == 2) || (i == 4)             ))
			|| ([pattern isEqualToString:@"WPWPWPHL"]     && ((i == 2) || (i == 4) || (i == 6) ))
			)
		{
			#ifdef DEBUG
				NSLog(@"%d rProv7", i);
			#endif
			obj = [nodeArray objectAtIndex:i];
			rProv7 = [NSMutableArray array];
			nodeList = [obj getElementsByTagName:@"TD"];
			for (j = 1; j < 8; j++) {
				childObj = [nodeList item:j];
				rProv = [[childObj innerText] stringByExtractingNumber];
				if ([rProv length] > 0) {
					[rProv7 addObject:rProv];
				} else {
					[rProv7 addObject:@"n/a"];
				}
			}
		}

		// tempH7
		if (   ([pattern isEqualToString:@"WPHL"]         &&  (i == 3)                          )
			|| ([pattern isEqualToString:@"WPHLWP"]       &&  (i == 3)                          )
			|| ([pattern isEqualToString:@"WPHLWPHL"]     && ((i == 3) || (i == 7)             ))
			|| ([pattern isEqualToString:@"WPHLWPWP"]     &&  (i == 3)                          )
			|| ([pattern isEqualToString:@"WPHLWPWPHL"]   && ((i == 3) || (i == 9)             ))
			|| ([pattern isEqualToString:@"WPHLWPHLWP"]   && ((i == 3) || (i == 7)             ))
			|| ([pattern isEqualToString:@"WPHLWPHLWPHL"] && ((i == 3) || (i == 7) || (i == 11)))
			|| ([pattern isEqualToString:@"WPHLWPHLWPHLWPHL"] && ((i == 3) || (i == 7) || (i == 11) || (i == 15)))
			|| ([pattern isEqualToString:@"WPWPHL"]       &&  (i == 5)                          )
			|| ([pattern isEqualToString:@"WPWPWPHL"]     &&  (i == 7)                          )
			)
		{
			#ifdef DEBUG
				NSLog(@"%d tempH7 a", i);
			#endif
			obj = [nodeArray objectAtIndex:i];
			tempH7 = [NSMutableArray array];
			nodeList = [obj getElementsByTagName:@"TD"];
			for (j = 1; j < 8; j++) {
				childObj = [nodeList item:j];
				tempH = [[childObj innerText] stringByExtractingNumber];
				if ([tempH length] > 0) {
					[tempH7 addObject:tempH];
				} else {
					[tempH7 addObject:@"n/a"];
				}
			}
		}
		if (   ([pattern isEqualToString:@"WPHLWP"]       &&  (i == 6)                          )
			|| ([pattern isEqualToString:@"WPHLWPWP"]     && ((i == 6) || (i == 8)             ))
			|| ([pattern isEqualToString:@"WPHLWPWPHL"]   &&  (i == 6)                          )
			|| ([pattern isEqualToString:@"WPHLWPHLWP"]   &&  (i == 10)                         )
			|| ([pattern isEqualToString:@"WPWPHL"]       &&  (i == 2)                          )
			|| ([pattern isEqualToString:@"WPWPWPHL"]     && ((i == 2) || (i == 4)             ))
			)
		{
			#ifdef DEBUG
				NSLog(@"%d tempH7 b", i);
			#endif
			tempH7 = [NSMutableArray array];
			for (j = 1; j < 8; j++) {
				[tempH7 addObject:@"n/a"];
			}
		}

		// tempL7
		if (   ([pattern isEqualToString:@"WPHL"]         &&  (i == 4)                          )
			|| ([pattern isEqualToString:@"WPHLWP"]       &&  (i == 4)                          )
			|| ([pattern isEqualToString:@"WPHLWPHL"]     && ((i == 4) || (i == 8)             ))
			|| ([pattern isEqualToString:@"WPHLWPWP"]     &&  (i == 4)                          )
			|| ([pattern isEqualToString:@"WPHLWPWPHL"]   && ((i == 4) || (i == 10)            ))
			|| ([pattern isEqualToString:@"WPHLWPHLWP"]   && ((i == 4) || (i == 8)             ))
			|| ([pattern isEqualToString:@"WPHLWPHLWPHL"] && ((i == 4) || (i == 8) || (i == 12)))
			|| ([pattern isEqualToString:@"WPHLWPHLWPHLWPHL"] && ((i == 4) || (i == 8) || (i == 12) || (i == 16)))
			|| ([pattern isEqualToString:@"WPWPHL"]       &&  (i == 6)                          )
			|| ([pattern isEqualToString:@"WPWPWPHL"]     &&  (i == 8)                          )
			)
		{
			#ifdef DEBUG
				NSLog(@"%d tempL7 a", i);
			#endif
			obj = [nodeArray objectAtIndex:i];
			tempL7 = [NSMutableArray array];
			nodeList = [obj getElementsByTagName:@"TD"];
			for (j = 1; j < 8; j++) {
				childObj = [nodeList item:j];
				tempH = [[childObj innerText] stringByExtractingNumber];
				if ([tempL length] > 0) {
					[tempL7 addObject:tempH];
				} else {
					[tempL7 addObject:@"n/a"];
				}
			}
		}
		if (   ([pattern isEqualToString:@"WPHLWP"]       &&  (i == 6)                          )
			|| ([pattern isEqualToString:@"WPHLWPWP"]     && ((i == 6) || (i == 8)             ))
			|| ([pattern isEqualToString:@"WPHLWPWPHL"]   &&  (i == 6)                          )
			|| ([pattern isEqualToString:@"WPHLWPHLWP"]   &&  (i == 10)                         )
			|| ([pattern isEqualToString:@"WPWPHL"]       &&  (i == 2)                          )
			|| ([pattern isEqualToString:@"WPWPWPHL"]     && ((i == 2) || (i == 4)             ))
			)
		{
			#ifdef DEBUG
				NSLog(@"%d tempL7 b", i);
			#endif
			tempL7 = [NSMutableArray array];
			for (j = 1; j < 8; j++) {
				[tempL7 addObject:@"n/a"];
			}
		}

		if (   ([pattern isEqualToString:@"WPHL"]         &&  (i == 4)                          )
			|| ([pattern isEqualToString:@"WPHLWP"]       && ((i == 4) || (i == 6)             ))
			|| ([pattern isEqualToString:@"WPHLWPHL"]     && ((i == 4) || (i == 8)             ))
			|| ([pattern isEqualToString:@"WPHLWPWP"]     && ((i == 4) || (i == 6) || (i == 8) ))
			|| ([pattern isEqualToString:@"WPHLWPWPHL"]   && ((i == 4) || (i == 6) || (i == 12)))
			|| ([pattern isEqualToString:@"WPHLWPHLWP"]   && ((i == 4) || (i == 8) || (i == 10)))
			|| ([pattern isEqualToString:@"WPHLWPHLWPHL"] && ((i == 4) || (i == 8) || (i == 12)))
			|| ([pattern isEqualToString:@"WPHLWPHLWPHLWPHL"] && ((i == 4) || (i == 8) || (i == 12) || (i == 16)))
			|| ([pattern isEqualToString:@"WPWPHL"]       && ((i == 2) || (i == 6)             ))
			|| ([pattern isEqualToString:@"WPWPWPHL"]     && ((i == 2) || (i == 4) || (i == 8) ))
			)
		{
			sectionData = [NSMutableArray array];
			for (j = 0; j < 7; j++) {
				[sectionData addObject:[NSArray arrayWithObjects:
					[date7 objectAtIndex:j],
					[weatherIcon7 objectAtIndex:j],
					[rProv7 objectAtIndex:j],
					[tempL7 objectAtIndex:j],
					[tempH7 objectAtIndex:j],
					nil]];
			}
			[loadedData addObject:sectionData];
		}
	}
	return loadedData;
}

@end
